@@ -15,19 +15,11 @@
 """
 An order preserving multidict implementation
 """
-from __future__ import absolute_import
-
-try:
-    from collections import MutableMapping
-except ImportError:
-    from UserDict import DictMixin
-
-    class MutableMapping(DictMixin):
-        """
-        Emulate MutableMapping in Python<2.6
-        """
-
+from collections.abc import MutableMapping
 from itertools import repeat
+from typing import Any
+from typing import Dict
+from typing import Set
 
 
 class MultiDict(MutableMapping):
@@ -53,7 +45,6 @@ class MultiDict(MutableMapping):
     """
 
     setdefault = MutableMapping.setdefault
-    __contains__ = MutableMapping.__contains__
 
     def __init__(self, *args, **kwargs):
         """
@@ -86,8 +77,10 @@ class MultiDict(MutableMapping):
 
         """
         if len(args) > 1:
-            raise TypeError("%s expected at most 2 arguments, got %d" %
-                            (self.__class__.__name__,  1 + len(args)))
+            raise TypeError(
+                "%s expected at most 2 arguments, got %d"
+                % (self.__class__.__name__, 1 + len(args))
+            )
         self.clear()
         self.update(*args, **kwargs)
 
@@ -113,7 +106,7 @@ class MultiDict(MutableMapping):
             >>> d
             MultiDict([('b', 3)])
         """
-        _order = [(k, v) for k, v in self._order if k != key]
+        _order = [(k, v) for k, v in self._order if k != key]  # type: ignore
         _order.append((key, value))
         self._order = _order
         self._dict[key] = [value]
@@ -163,6 +156,7 @@ class MultiDict(MutableMapping):
         """
         return self.__class__(self)
 
+    @classmethod
     def fromkeys(cls, seq, value=None):
         """\
         Create a new MultiDict with keys from seq and values set to value.
@@ -182,7 +176,6 @@ class MultiDict(MutableMapping):
 
         """
         return cls(zip(seq, repeat(value)))
-    fromkeys = classmethod(fromkeys)
 
     def items(self):
         """\
@@ -193,7 +186,7 @@ class MultiDict(MutableMapping):
             >>> list(d.items())
             [('a', 1), ('b', 3)]
         """
-        seen = set()
+        seen: Set[Any] = set()
         for k, v in self._order:
             if k in seen:
                 continue
@@ -284,8 +277,9 @@ class MultiDict(MutableMapping):
             KeyError: 'a'
         """
         if len(args) > 1:
-            raise TypeError("pop expected at most 2 arguments, got %d" %
-                            (1 + len(args)))
+            raise TypeError(
+                "pop expected at most 2 arguments, got %d" % (1 + len(args))
+            )
         try:
             value = self._dict[key].pop(0)
         except (KeyError, IndexError):
@@ -317,7 +311,7 @@ class MultiDict(MutableMapping):
         try:
             key = self._order[-1][0]
         except IndexError:
-            raise KeyError('popitem(): dictionary is empty')
+            raise KeyError("popitem(): dictionary is empty")
         return key, self.pop(key)
 
     def update(self, *args, **kwargs):
@@ -346,7 +340,7 @@ class MultiDict(MutableMapping):
             >>> d.update([('hobby', 'shopping'),
             ...           ('hobby', 'pressing wild flowers')])
             >>> d
-            MultiDict([('name', 'eric'), ('occupation', 'lumberjack'), ('hobby', 'shopping'), ('hobby', 'pressing wild flowers')])
+            MultiDict([('name', 'eric'), ...])
 
         or keyword arguments:
 
@@ -356,8 +350,9 @@ class MultiDict(MutableMapping):
 
         """
         if len(args) > 1:
-            raise TypeError("expected at most 1 argument, got %d" %
-                            (1 + len(args),))
+            raise TypeError(
+                "expected at most 1 argument, got %d" % (1 + len(args),)
+            )
         if args:
             other = args[0]
         else:
@@ -371,8 +366,9 @@ class MultiDict(MutableMapping):
         except that new keys are added to old keys.
         """
         if len(args) > 1:
-            raise TypeError("expected at most 1 argument, got %d"\
-                            (1 + len(args),))
+            raise TypeError(
+                "expected at most 1 argument, got %d", (1 + len(args),)
+            )
         if args:
             other = args[0]
         else:
@@ -402,8 +398,9 @@ class MultiDict(MutableMapping):
 
         if replace:
             replaced = set(k for k, v in items if k in self._dict)
-            self._order = [(k, v)
-                           for (k, v) in self._order if k not in replaced]
+            self._order = [
+                (k, v) for (k, v) in self._order if k not in replaced
+            ]
             for key in replaced:
                 self._dict[key] = []
 
@@ -415,7 +412,7 @@ class MultiDict(MutableMapping):
         """
         ``__repr__`` representation of object
         """
-        return '%s(%r)' % (self.__class__.__name__, list(self.allitems()))
+        return "%s(%r)" % (self.__class__.__name__, list(self.allitems()))
 
     def __str__(self):
         """
@@ -430,12 +427,11 @@ class MultiDict(MutableMapping):
         return len(self._dict)
 
     def __eq__(self, other):
-        return isinstance(other, MultiDict) \
-            and self._order == other._order
+        return isinstance(other, MultiDict) and self._order == other._order
 
     def __ne__(self, other):
         return not (self == other)
 
     def clear(self):
         self._order = []
-        self._dict = {}
+        self._dict: Dict[Any, Any] = {}
