@@ -12,8 +12,6 @@
 #     See the License for the specific language governing permissions and
 #     limitations under the License.
 #
-from __future__ import absolute_import
-
 import imp
 import sys
 from random import randint
@@ -33,6 +31,9 @@ class Options(dict):
     def __setattr__(self, key, value):
         self[key] = value
 
+    def copy(self):
+        return self.__class__(super().copy())
+
     def update_from_file(self, path, load_all=False):
         """
         Update the instance with any symbols found in the python source file at
@@ -42,7 +43,7 @@ class Options(dict):
         :param load_all: If true private symbols will also be loaded into the
                          options object.
         """
-        module_name = 'fresco_options_%d%d' % (time(), randint(0, sys.maxsize))
+        module_name = "fresco_options_%d%d" % (time(), randint(0, sys.maxsize))
         module = imp.load_source(module_name, path)
         self.update_from_object(module, load_all)
         del sys.modules[module_name]
@@ -56,8 +57,7 @@ class Options(dict):
         if load_all:
             symbols = dir(ob)
         else:
-            symbols = getattr(ob, '__all__', None)
+            symbols = getattr(ob, "__all__", None)
             if symbols is None:
-                symbols = [name for name in dir(ob)
-                                if not name.startswith('_')]
+                symbols = [name for name in dir(ob) if not name.startswith("_")]
         self.update((s, getattr(ob, s)) for s in symbols)
