@@ -19,79 +19,36 @@ fresco.request
 The :class:`Request` class models an incoming HTTP request, allowing access to
 HTTP headers and request data (eg query string or submitted form data).
 """
-from __future__ import absolute_import
-
+from urllib.parse import ParseResult
+from urllib.parse import quote
+from urllib.parse import urlparse
+from urllib.parse import urlunparse
+from typing import Dict
+from typing import Optional
+import datetime
 import json
 import posixpath
 import re
 
 from fresco import exceptions
-from fresco.compat import (ustr, quote, urlparse, urlunparse,
-                           ParseResult)
-from fresco.util.wsgi import environ_to_unicode
-
+from fresco.cookie import parse_cookie_header
+from fresco.multidict import MultiDict
+from fresco.util.http import FileUpload
+from fresco.util.http import get_body_bytes
+from fresco.util.http import get_content_type_info
+from fresco.util.http import parse_post
+from fresco.util.http import parse_querystring
+from fresco.util.urls import normpath
+from fresco.util.urls import make_query
+from fresco.util.wsgi import environ_to_str
 import fresco
 
-from .multidict import MultiDict
-from .cookie import parse_cookie_header
-from .util.urls import normpath, make_query
-
-__all__ = 'Request', 'currentrequest'
+__all__ = "Request", "currentrequest"
 
 KB = 1024
 MB = 1024 * KB
 
-# List of standard request header field names
-# This is taken from the list available at
-# http://en.wikipedia.org/wiki/List_of_HTTP_header_fields which combines
-# RFC specified headers with commonly used non-standard header names
-HEADER_NAMES = {
-    'accept': 'Accept',
-    'accept_charset': 'Accept-Charset',
-    'accept_encoding': 'Accept-Encoding',
-    'accept_language': 'Accept-Language',
-    'accept_datetime': 'Accept-Datetime',
-    'authorization': 'Authorization',
-    'cache_control': 'Cache-Control',
-    'connection': 'Connection',
-    'cookie': 'Cookie',
-    'content_length': 'Content-Length',
-    'content_md5': 'Content-MD5',
-    'content_type': 'Content-Type',
-    'date': 'Date',
-    'expect': 'Expect',
-    'from': 'From',
-    'host': 'Host',
-    'if_match': 'If-Match',
-    'if_modified_since': 'If-Modified-Since',
-    'if_none_match': 'If-None-Match',
-    'if_range': 'If-Range',
-    'if_unmodified_since': 'If-Unmodified-Since',
-    'max_forwards': 'Max-Forwards',
-    'origin': 'Origin',
-    'pragma': 'Pragma',
-    'proxy_authorization': 'Proxy-Authorization',
-    'range': 'Range',
-    'referer': 'Referer',
-    'te': 'TE',
-    'user_agent': 'User-Agent',
-    'upgrade': 'Upgrade',
-    'via': 'Via',
-    'warning': 'Warning',
-    'common': 'Common',
-    'field': 'Field',
-    'x_requested_with': 'X-Requested-With',
-    'dnt': 'DNT',
-    'x_forwarded_for': 'X-Forwarded-For',
-    'x_forwarded_host': 'X-Forwarded-Host',
-    'x_forwarded_host': 'X-Forwarded-Host',
-    'x_forwarded_proto': 'X-Forwarded-Proto',
-    'front_end_https': 'Front-End-Https',
-    'x_http_method_override': 'X-Http-Method-Override',
-    'x_att_deviceid': 'X-ATT-DeviceId',
-    'x_wap_profile': 'X-Wap-Profile',
-    'proxy_connection': 'Proxy-Connection',
-}
+_marker = object()
 
 
 class Request(object):
@@ -107,6 +64,9 @@ class Request(object):
     #: Maximum size for multipart/form-data encoded post data
     MAX_MULTIPART_SIZE = 2 * MB
 
+    #: Support broken IE content-disposition escaping
+    IE_CONTENT_DISPOSITION_WORKAROUND = True
+
     _body_bytes = None
     _parsed_content_type = None
     _form = None
@@ -114,46 +74,54 @@ class Request(object):
     _query = None
     _cookies = None
     _parsed_url = None
+    _now = None
+
+    STATE_ENV_KEY = "fresco._request_state"
 
     #: WSGI key for the session variable. The default value is configured for
     #: use with `beaker <http://beaker.groovie.org/>`_
-    SESSION_ENV_KEY = 'beaker.session'
+    SESSION_ENV_KEY = "beaker.session"
 
     #: The WSGI environ dict
-    environ = None
+    environ: Dict
 
     #: Encoding used to decode WSGI parameters, notably PATH_INFO and form data
-    charset = fresco.DEFAULT_CHARSET
+    default_charset = fresco.DEFAULT_CHARSET
 
     #: The decoder class to use for JSON request payloads
     json_decoder_class = json.JSONDecoder
 
-    def __new__(
-        cls,
-        environ,
-        parse_content_type=re.compile(r'\s*(?:.*);\s*charset=([\w\d\-]+)\s*$')
-    ):
-        try:
-            return environ['fresco.request']
-        except KeyError:
-            request = object.__new__(cls)
-            request.environ = environ
-            request.environ['fresco.request'] = request
-            if 'CONTENT_TYPE' in environ:
-                match = parse_content_type.match(environ['CONTENT_TYPE'])
-                if match:
-                    request.charset = match.group(1)
-            return request
+    def __init__(self, environ):
+        if self.STATE_ENV_KEY in environ:
+            self.__dict__ = environ[self.STATE_ENV_KEY]
+        else:
+            environ[self.STATE_ENV_KEY] = self.__dict__
+        self.environ = environ
 
     def __str__(self):
         """
         Return a useful text representation of the request
         """
-        return "<%s %s %s>" % (
-                self.__class__.__name__,
-                self.method,
-                self.url,
-        )
+        return "<%s %s %s>" % (self.__class__.__name__, self.method, self.url)
+
+    @property
+    def charset(  # type: ignore
+        self, parse_charset=re.compile(r";\s*charset=([\w\d\-]+)").search
+    ) -> str:
+        try:
+            return self.environ["fresco.request.charset"]
+        except KeyError:
+            r = self.default_charset
+            ct = self.environ.get("CONTENT_TYPE")
+            if ct is not None:
+                match = parse_charset(ct)
+                if match is not None:
+                    r = self.environ["fresco.request.charset"] = match.group(1)
+        return r
+
+    @charset.setter
+    def charset(self, value):
+        self.environ["fresco.request.charset"] = value
 
     @property
     def form(self):
@@ -165,18 +133,20 @@ class Request(object):
         the ``query`` property.
         """
         if self._form is None:
-            if self.environ['REQUEST_METHOD'] in ('PUT', 'POST'):
+            if self.environ["REQUEST_METHOD"] in ("PUT", "POST"):
                 self._form = MultiDict(
-                    parse_post(self.environ,
-                               self.environ['wsgi.input'],
-                               self.charset,
-                               self.MAX_SIZE,
-                               self.MAX_MULTIPART_SIZE)
+                    parse_post(
+                        self.environ,
+                        self.environ["wsgi.input"],
+                        self.charset,
+                        self.MAX_SIZE,
+                        self.MAX_MULTIPART_SIZE,
+                        ie_workaround=self.IE_CONTENT_DISPOSITION_WORKAROUND,
+                    )
                 )
             else:
-                query = environ_to_unicode(
-                            self.environ.get('QUERY_STRING', ''))
-                self._form = MultiDict(parse_querystring(query, self.charset))
+                data = environ_to_str(self.environ.get("QUERY_STRING", ""))
+                self._form = MultiDict(parse_querystring(data, self.charset))
         return self._form
 
     @property
@@ -207,8 +177,9 @@ class Request(object):
             return body.decode(encoding)
         except UnicodeDecodeError:
             raise exceptions.RequestParseError(
-                "Payload contains data that could not be decoded using " +
-                encoding)
+                "Payload contains data that could not be decoded using "
+                + encoding
+            )
 
     def get_json(self, *args, **kwargs):
         """
@@ -231,12 +202,15 @@ class Request(object):
         Return a MultiDict of all ``FileUpload`` objects available.
         """
         if self._files is None:
-            self._files = MultiDict((k, v) for k, v in self.form.allitems()
-                                           if isinstance(v, FileUpload))
+            self._files = MultiDict(
+                (k, v)
+                for k, v in self.form.allitems()
+                if isinstance(v, FileUpload)
+            )
         return self._files
 
     @property
-    def query(self):
+    def query(self, environ_to_str=environ_to_str) -> MultiDict:  # type: ignore
         """
         Return a ``MultiDict`` of any querystring submitted data.
 
@@ -254,18 +228,17 @@ class Request(object):
         request method used.
         """
         if self._query is None:
-            query = environ_to_unicode(self.environ.get('QUERY_STRING', ''))
+            query = environ_to_str(self.environ.get("QUERY_STRING", ""))
             self._query = MultiDict(parse_querystring(query, self.charset))
 
         return self._query
 
-    def __getitem__(self, key):
+    def __getitem__(self, key, _marker=_marker):
         """
         Return the value of ``key`` from submitted form values.
         """
-        marker = []
-        v = self.get(key, marker)
-        if v is marker:
+        v = self.get(key, _marker)
+        if v is _marker:
             raise KeyError(key)
         return v
 
@@ -281,12 +254,45 @@ class Request(object):
         """
         return self.form.getlist(key)
 
+    def getint(self, key, default=_marker) -> int:
+        """
+        Return the named key, converted to an integer.
+
+        If the key is not present or could not be converted to an int and no
+        default is specified, :class:`~fresco.exceptions.BadRequest` will be
+        raised.
+        """
+        v = self.get(key, _marker)
+        if v is _marker:
+            if default is _marker:
+                raise exceptions.BadRequest()
+            else:
+                return default
+        try:
+            return int(v)
+        except Exception:
+            if default is _marker:
+                raise exceptions.BadRequest()
+            else:
+                return default
+
     def __contains__(self, key):
         """
         Return ``True`` if ``key`` is in the submitted form values
         """
         return key in self.form
 
+    @property
+    def now(self, utcnow=datetime.datetime.now, utc=datetime.timezone.utc):  # type: ignore
+        """
+        Return a timezone-aware UTC datetime instance. The value returned is
+        guaranteed to be constant throughout the lifetime of the request.
+        """
+        if self._now:
+            return self._now
+        self._now = utcnow().replace(tzinfo=utc)
+        return self._now
+
     @property
     def cookies(self):
         """
@@ -310,7 +316,8 @@ class Request(object):
         """
         if self._cookies is None:
             self._cookies = MultiDict(
-                parse_cookie_header(self.environ.get('HTTP_COOKIE', '')))
+                parse_cookie_header(self.environ.get("HTTP_COOKIE", ""))
+            )
         return self._cookies
 
     def get_header(self, name, default=None):
@@ -332,8 +339,7 @@ class Request(object):
         not case sensitive.
         """
         return self.environ.get(
-            'HTTP_' + name.upper().replace('-', '_'),
-            default
+            "HTTP_" + name.upper().replace("-", "_"), default
         )
 
     @property
@@ -341,8 +347,10 @@ class Request(object):
         """\
         Return the path component of the requested URL
         """
-        return environ_to_unicode(self.environ.get('SCRIPT_NAME', '') +
-                                  self.environ.get('PATH_INFO', ''))
+        return environ_to_str(
+            self.environ.get("SCRIPT_NAME", "")
+            + self.environ.get("PATH_INFO", "")
+        )
 
     @property
     def url(self):
@@ -368,9 +376,16 @@ class Request(object):
         """
         scheme, netloc, path, params, query, frag = self.parsed_url
 
-        return urlunparse((scheme, netloc,
-                           quote(self.script_name.encode(self.charset)), '',
-                           '', ''))
+        return urlunparse(
+            (
+                scheme,
+                netloc,
+                quote(self.script_name.encode(self.charset)),
+                "",
+                "",
+                "",
+            )
+        )
 
     @property
     def parsed_url(self):
@@ -398,46 +413,49 @@ class Request(object):
         script_name = quote(self.script_name.encode(self.charset))
         path_info = quote(self.path_info.encode(self.charset))
         query_string = self.query_string
-        scheme = environ_to_unicode(env('wsgi.url_scheme', 'http'))
+        scheme = environ_to_str(env("wsgi.url_scheme", "http"))
 
         try:
-            host = environ_to_unicode(self.environ['HTTP_HOST'])
-            if ':' in host:
-                host, port = host.split(':', 1)
+            host = environ_to_str(self.environ["HTTP_HOST"])
+            if ":" in host:
+                host, port = host.split(":", 1)
             else:
-                if scheme == 'https':
-                    port = '443'
+                if scheme == "https":
+                    port = "443"
                 else:
-                    port = '80'
+                    port = "80"
         except KeyError:
-            host = environ_to_unicode(self.environ['SERVER_NAME'])
-            port = environ_to_unicode(self.environ['SERVER_PORT'])
+            host = environ_to_str(self.environ["SERVER_NAME"])
+            port = environ_to_str(self.environ["SERVER_PORT"])
 
-        if (scheme == 'http' and port == '80') \
-                or (scheme == 'https' and port == '443'):
+        if (scheme == "http" and port == "80") or (
+            scheme == "https" and port == "443"
+        ):
             netloc = host
         else:
-            netloc = host + ':' + port
+            netloc = host + ":" + port
 
-        self._parsed_url = ParseResult(scheme,
-                                       netloc,
-                                       script_name + path_info,
-                                       '',  # Params
-                                       query_string,
-                                       '',  # Fragment
-                                       )
+        self._parsed_url = ParseResult(
+            scheme,
+            netloc,
+            script_name + path_info,
+            "",  # Params
+            query_string or "",
+            "",  # Fragment
+        )
         return self._parsed_url
 
     @property
-    def path_info(self):
+    def path_info(self, environ_to_str=environ_to_str) -> str:  # type: ignore
         """
-        The PATH_INFO value as a unicode string
+        The PATH_INFO value as a string
 
         Note that PATH_INFO is already unquoted by the server
         """
         try:
-            return environ_to_unicode(self.environ.get('PATH_INFO', ''),
-                                      self.charset)
+            return environ_to_str(
+                self.environ.get("PATH_INFO", ""), self.charset
+            )
         except UnicodeDecodeError:
             raise exceptions.BadRequest
 
@@ -449,47 +467,50 @@ class Request(object):
         Note that SCRIPT_NAME is already unquoted by the server
         """
         try:
-            return environ_to_unicode(self.environ.get('SCRIPT_NAME', ''),
-                                      self.charset)
+            return environ_to_str(
+                self.environ.get("SCRIPT_NAME", ""), self.charset
+            )
         except UnicodeDecodeError:
             raise exceptions.BadRequest
 
     @property
-    def query_string(self):
+    def query_string(  # type: ignore
+        self, environ_to_str=environ_to_str,
+    ) -> Optional[str]:
         """\
         The QUERY_STRING value as a unicode string
         """
-        try:
-            return environ_to_unicode(self.environ.get("QUERY_STRING"))
-        except AttributeError:
-            return None
+        s = self.environ.get("QUERY_STRING")
+        if s is None:
+            return s
+        return environ_to_str(s)
 
     @property
     def referrer(self):
         """
         Return the HTTP referer header, or ``None`` if this is not available.
         """
-        try:
-            return environ_to_unicode(self.environ.get("HTTP_REFERER"))
-        except AttributeError:
-            return None
+        s = self.environ.get("HTTP_REFERER")
+        if s is None:
+            return s
+        return environ_to_str(s)
 
     @property
     def method(self):
         """
         Return the HTTP method used for the request, eg ``GET`` or ``POST``.
         """
-        return environ_to_unicode(self.environ["REQUEST_METHOD"])
+        return self.environ["REQUEST_METHOD"]
 
     @property
     def remote_addr(self):
         """
         Return the remote address of the client
         """
-        try:
-            return environ_to_unicode(self.environ.get("REMOTE_ADDR"))
-        except AttributeError:
-            return None
+        s = self.environ.get("REMOTE_ADDR")
+        if s is None:
+            return s
+        return environ_to_str(s)
 
     @property
     def session(self):
@@ -506,11 +527,22 @@ class Request(object):
         """
         Return ``True`` if the request is served over a secure connection.
         """
-        return self.environ['wsgi.url_scheme'] == 'https'
+        return self.environ["wsgi.url_scheme"] == "https"
 
-    def make_url(self, scheme=None, netloc=None, path=None, parameters=None,
-                 query=None, query_add=None, query_replace=None, fragment=None,
-                 SCRIPT_NAME=None, PATH_INFO=None, **kwargs):
+    def make_url(
+        self,
+        scheme=None,
+        netloc=None,
+        path=None,
+        parameters=None,
+        query=None,
+        query_add=None,
+        query_replace=None,
+        fragment=None,
+        SCRIPT_NAME=None,
+        PATH_INFO=None,
+        **kwargs,
+    ):
         r"""\
         Make a new URL based on the current URL, replacing any of the six
         URL elements (scheme, netloc, path, parameters, query or fragment). The
@@ -523,7 +555,9 @@ class Request(object):
         :param parameters:  RFC2396 path parameters (see also the stdlib
                             urlparse module)
         :param fragment:    The URL fragment
-        :param query:       Query data, as a list of tuples, a dict or a string
+        :param query:       Query data, as a list of tuples, a dict or a string.
+                            Any query parameters present in the current url
+                            will be removed.
         :param query_add:   Query parameters to add. If the current url already
                             contains query parameters with the same name, these
                             will be supplemented with the new values.
@@ -539,8 +573,7 @@ class Request(object):
         :rtype: str
 
         All arguments (other than query related arguments, see examples below)
-        may be byte strings or unicode strings (native strings in python 3).
-        Where necessary values will be converted to utf-8 encoded byte strings.
+        must be strings (not bytes).
 
         Examples:
 
@@ -610,17 +643,13 @@ class Request(object):
         These arguments take a dict, a list of ``(key, value)`` tuples, or any
         other type convertable to a MultiDict.
         """
-        url = []
         parsed_url = self.parsed_url
 
         if path is not None:
-            if not isinstance(path, bytes):
-                path = path.encode('utf-8')
             path = quote(path)
 
-            if path[0] != '/':
-                path = posixpath.join(
-                        posixpath.dirname(parsed_url[2]), path)
+            if path[0] != "/":
+                path = posixpath.join(posixpath.dirname(parsed_url[2]), path)
                 path = posixpath.normpath(path)
 
         elif SCRIPT_NAME is not None or PATH_INFO is not None:
@@ -628,49 +657,51 @@ class Request(object):
                 SCRIPT_NAME = self.script_name
             if PATH_INFO is None:
                 PATH_INFO = self.path_info
-            path = SCRIPT_NAME + PATH_INFO
-            if not isinstance(path, bytes):
-                path = path.encode('utf-8')
-            path = quote(path)
+            path = quote(SCRIPT_NAME + PATH_INFO)
 
         else:
             path = parsed_url[2]
 
-        if query is not None:
-            if isinstance(query, ustr):
-                # Unicode queries should be pre-encoded and thus contain ascii
-                # characters only
-                query = query.encode('ASCII')
-            elif isinstance(query, bytes):
-                if kwargs:
-                    if query:
-                        query += ';'
-                    query += make_query(kwargs)
+        query_dict = None
+        if query_add or query_replace or kwargs:
+            if query_add or query_replace:
+                query = query if query is not None else parsed_url[4]
+            else:
+                query = query if query is not None else MultiDict()
 
-        if (query_add or query_replace) and isinstance(query, bytes):
-            query = MultiDict(parse_querystring(query.decode('ASCII')))
+            if isinstance(query, str):
+                query_dict = MultiDict(parse_querystring(query))
+            elif isinstance(query, MultiDict):
+                query_dict = query
+            else:
+                query_dict = MultiDict(query)
 
-        if query_add is not None:
-            query = query if query is not None else self.query.copy()
-            query.extend(query_add)
+            if query_add is not None:
+                query_dict.extend(query_add)
 
-        if query_replace is not None:
-            query = query if query is not None else self.query.copy()
-            query.update(query_replace)
+            if query_replace is not None:
+                query_dict.update(query_replace)
 
-        if query and not isinstance(query, bytes):
-            query = make_query(query, **kwargs)
+            if kwargs:
+                query_dict.update(kwargs)
 
-        url = [scheme if scheme is not None else parsed_url[0],
-               netloc if netloc is not None else parsed_url[1],
-               path if path is not None else parsed_url[2],
-               parameters if parameters is not None else parsed_url[3],
-               query,
-               fragment]
+            query = make_query(query_dict)
+
+        elif query is not None and not isinstance(query, str):
+            query = make_query(query)
+
+        url = (
+            scheme if scheme is not None else parsed_url[0],
+            netloc if netloc is not None else parsed_url[1],
+            path if path is not None else parsed_url[2],
+            parameters if parameters is not None else parsed_url[3],
+            query,
+            fragment,
+        )
 
         return urlunparse(url)
 
-    def resolve_url(self, url, relative='app'):
+    def resolve_url(self, url, relative="app"):
         """
         Resolve a partially qualified URL in context of the current request.
 
@@ -717,41 +748,46 @@ class Request(object):
         """
         env = self.environ.get
 
-        if '://' not in url:
-            scheme = env('wsgi.url_scheme', 'http')
+        if "://" not in url:
+            scheme = env("wsgi.url_scheme", "http")
 
-            if scheme == 'https':
-                port = ':' + env('SERVER_PORT', '443')
+            if scheme == "https":
+                port = ":" + env("SERVER_PORT", "443")
             else:
-                port = ':' + env('SERVER_PORT', '80')
+                port = ":" + env("SERVER_PORT", "80")
 
-            if scheme == 'http' and port == ':80' \
-               or scheme == 'https' and port == ':443':
-                port = ''
+            if (
+                scheme == "http"
+                and port == ":80"
+                or scheme == "https"
+                and port == ":443"
+            ):
+                port = ""
 
             parsed = urlparse(url)
-            path_info = env('PATH_INFO', '')
-            script_name = env('SCRIPT_NAME', '')
+            path_info = env("PATH_INFO", "")
+            script_name = env("SCRIPT_NAME", "")
 
-            if path_info[-1] != '/':
-                path_info = path_info[:path_info.rfind('/') + 1]
+            if path_info[-1] != "/":
+                path_info = path_info[: path_info.rfind("/") + 1]
 
-            if relative == 'app':
+            if relative == "app":
                 path = posixpath.join(quote(path_info), parsed[2])
                 path = quote(script_name) + normpath(path)
 
             else:
-                path = posixpath.join(quote(script_name) + path_info,
-                                      parsed[2])
+                path = posixpath.join(quote(script_name) + path_info, parsed[2])
 
-            url = urlunparse((
-                env('wsgi.url_scheme', ''),
-                env('HTTP_HOST', env('SERVER_NAME', '') + port),
-                normpath(path),
-                parsed[3],
-                parsed[4],
-                parsed[5],
-            ))
+            url = urlunparse(
+                (
+                    env("wsgi.url_scheme", ""),
+                    env("HTTP_HOST", env("SERVER_NAME", "") + port),
+                    normpath(path),
+                    parsed[3],
+                    parsed[4],
+                    parsed[5],
+                )
+            )
         return url
 
 
@@ -764,6 +800,3 @@ def currentrequest():
         return fresco.context.request
     except AttributeError:
         return None
-
-from .util.http import (FileUpload, parse_querystring, parse_post,
-                        get_content_type_info, get_body_bytes)
