@@ -1,5 +1,3 @@
-from __future__ import absolute_import
-
 # Copyright 2015 Oliver Cope
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
@@ -25,8 +23,7 @@ from fresco import FrescoApp
 from fresco.static import serve_static_file
 
 
-class TestServeStaticFile():
-
+class TestServeStaticFile:
     def setup(self):
 
         fh, tmpname = mkstemp()
@@ -38,17 +35,17 @@ class TestServeStaticFile():
         os.unlink(self.tmpname)
 
     def parse_lmh(self, last_modified):
-        return timegm(parsedate(last_modified))
+        return timegm(parsedate(last_modified))  # type: ignore
 
     def make_ims(self, since, tz=0):
-        return {'HTTP_IF_MODIFIED_SINCE': formatdate(since, tz)}
+        return {"HTTP_IF_MODIFIED_SINCE": formatdate(since, tz)}
 
     def test_sets_last_modified_header(self):
         with FrescoApp().requestcontext():
             response = serve_static_file(self.tmpname)
             response.content_iterator.close()
 
-        last_modified = response.get_header('Last-Modified')
+        last_modified = response.get_header("Last-Modified")
         assert self.parse_lmh(last_modified) == self.mtime
 
     def test_full_response_when_if_modified_before_mtime(self):
@@ -56,7 +53,7 @@ class TestServeStaticFile():
             response = serve_static_file(self.tmpname)
             response.content_iterator.close()
 
-        last_modified = response.get_header('Last-Modified')
+        last_modified = response.get_header("Last-Modified")
         assert response.status_code == 200
         assert self.parse_lmh(last_modified) == self.mtime
 
@@ -78,13 +75,15 @@ class TestServeStaticFile():
     def test_it_doesnt_fail_with_an_out_of_range_modified_since_value1(self):
         year = sys.maxsize
         with FrescoApp().requestcontext(
-                HTTP_IF_MODIFIED_SINCE='"Wed, 09 Feb %s 04:46:40 GMT"' % year):
+            HTTP_IF_MODIFIED_SINCE='"Wed, 09 Feb %s 04:46:40 GMT"' % year
+        ):
             response = serve_static_file(self.tmpname)
         assert response.status_code == 400
 
     def test_it_doesnt_fail_with_an_out_of_range_modified_since_value2(self):
         with FrescoApp().requestcontext(
-                HTTP_IF_MODIFIED_SINCE='"Wed, 01 Feb -2001 04:46:40 GMT"'):
+            HTTP_IF_MODIFIED_SINCE='"Wed, 01 Feb -2001 04:46:40 GMT"'
+        ):
             response = serve_static_file(self.tmpname)
         assert response.status_code == 400
 
@@ -93,20 +92,22 @@ class TestServeStaticFile():
             response = serve_static_file(self.tmpname)
             response.content_iterator.close()
 
-        assert re.match(r'^\w{3}, \d{1,2} \w{3} \d{4} \d\d:\d\d:\d\d GMT',
-                        response.get_header('last-modified'))
+        assert re.match(
+            r"^\w{3}, \d{1,2} \w{3} \d{4} \d\d:\d\d:\d\d GMT",
+            response.get_header("last-modified"),
+        )
 
     def test_filewrapper_used_if_present(self):
         with FrescoApp().requestcontext() as c:
             file_wrapper = Mock(return_value=[])
-            c.request.environ['wsgi.file_wrapper'] = file_wrapper
+            c.request.environ["wsgi.file_wrapper"] = file_wrapper
             serve_static_file(self.tmpname)
 
         assert file_wrapper.call_count == 1
 
     def test_it_returns_not_found(self):
         with FrescoApp().requestcontext():
-            response = serve_static_file(self.tmpname + 'x')
+            response = serve_static_file(self.tmpname + "x")
         assert response.status_code == 404
 
     def test_it_returns_forbidden(self):
