@@ -12,31 +12,23 @@
 #     See the License for the specific language governing permissions and
 #     limitations under the License.
 #
-from __future__ import absolute_import
-
 import pytest
 
-from fresco.util import security
+from fresco.util.security import check_equal_constant_time
 
 
 class TestSecurity(object):
-
     def test_check_equal_constant_time_returns_equality(self):
-        assert security.check_equal_constant_time('', '') is True
-        assert security.check_equal_constant_time('abcabcabc', 'abcabcabc') \
-                is True
+        assert check_equal_constant_time("", "") is True
+        assert check_equal_constant_time("abcabcabc", "abcabcabc") is True
 
     def test_check_equal_constant_time_returns_inequality(self):
-        assert security.check_equal_constant_time(' ', '') is False
-        assert security.check_equal_constant_time('abcabcabc', 'abcabcabx') \
-                is False
-        assert security.check_equal_constant_time('abcabcabc', 'abcabcabx') \
-                is False
-        assert security.check_equal_constant_time('abcabcabc', 'abcabcabde') \
-                is False
-        assert security.check_equal_constant_time('abcabcabc', 'abcabcab') \
-                is False
+        assert check_equal_constant_time(" ", "") is False
+        assert check_equal_constant_time("abcabcabc", "abcabcabx") is False
+        assert check_equal_constant_time("abcabcabc", "abcabcabx") is False
+        assert check_equal_constant_time("abcabcabc", "abcabcabde") is False
+        assert check_equal_constant_time("abcabcabc", "abcabcab") is False
 
     def test_check_equal_constant_time_raises_an_error_on_non_strings(self):
         with pytest.raises(TypeError):
-            security.check_equal_constant_time(None, None)
+            check_equal_constant_time(None, None)
